@@ -25,6 +25,7 @@ import sys, re, shutil
 import platform
 import gzip
 import subprocess
+from collections import defaultdict
 
 def cleanargs(rawargs):
     #TODO: Write docstring
@@ -79,7 +80,7 @@ def fastq2reads(infile,outfile,maxreads):
     if maxreads > -1:
         if tot > maxreads:
             break
-    if cnt==0: 
+    if cnt==0:
       h = line[1:] # strip off '@'
       #h = h.replace(' ','_')
       output.write(">%s" % h)
@@ -123,17 +124,17 @@ def fix_paired_headers_for_bwa(reads1,reads2):
       e = e[:i]+e[i+1:]
       f = f[:i]+f[i+1:]
     c.write(e+"\n")
-    d.write(f+"\n")      
+    d.write(f+"\n")
   except Exception as ex:
     a.close(); b.close(); c.close(); d.close()
     error(ex.args[0])
   a.close(); b.close(); c.close(); d.close()
-  
+
   if(os.path.exists(reads1)): os.remove(reads1)
   if(os.path.exists(reads2)): os.remove(reads2)
   os.rename(temp1, reads1)
   os.rename(temp2, reads2)
-  
+
   '''
   if platform.platform == 'win32':
 	  os.system("move %s %s" % (temp1,reads1))
@@ -143,6 +144,72 @@ def fix_paired_headers_for_bwa(reads1,reads2):
 	  os.system("mv %s %s" % (temp2, reads2))
   '''
 
+
+# checks for a match allowing 1 or 2 mismatches
+# if not a match returns -1,-1. If match occurs, returns 1 and the start index of match
+def bit_parallel_with_max_2_error(text, pattern, m):
+    S_table = defaultdict(int)
+    for i, c in enumerate(pattern):
+        S_table[c] |= 1 << i
+    R0 = 0
+    R1 = 0
+    R2 = 0
+    mask = 1 << (m - 1)
+    for j, c in enumerate(text):
+        S = S_table[c]
+        shR0 = (R0 << 1) | 1
+        shR1 = (R1 << 1) | 1
+        R0 = shR0 & S
+        R1 = ((R1 << 1) | 1) & S | shR0
+        R2 = ((R2 << 1) | 1) & S | shR1
+        # first if-statement commented because we have already checked for exact match using find() in mmfind()
+        #if R0 & mask:  #exact match
+            #return 1, j - m + 1
+        if R1 & mask:   # 1 mismatch
+            return 1, j - m + 1
+        if R2 & mask:   # 2 mismatches
+            return 1, j - m + 1
+    return -1,-1
+
+
+# checks for a match allowing 1 mismatch
+# if not a match returns -1,-1. If match occurs, returns 1 and the start index of match
+def bit_parallel_with_max_1_error(text, pattern, m):
+    S_table = defaultdict(int)
+    for i, c in enumerate(pattern):
+        S_table[c] |= 1 << i
+    R0 = 0
+    R1 = 0
+    mask = 1 << (m - 1)
+    for j, c in enumerate(text):
+        S = S_table[c]
+        shR0 = (R0 << 1) | 1
+        R0 = shR0 & S
+        R1 = ((R1 << 1) | 1) & S | shR0
+        # first if-statement commented because we have already checked for exact match using find() in mmfind()
+        #if R0 & mask:  #exact match
+            #return 1, j - m + 1
+        if R1 & mask:   # 1 mismatch
+            return 1, j - m + 1
+    return -1,-1
+
+
+# this function is a replacement for below mmfind() for speedup
+# it assumes the length of H is <32
+# find index of H[1..m] in G[1..n] with up to max (1 or 2) mismatches
+def mmfind(G,n,H,m,max): # lengths; assume n>m
+    a = G.find(H)
+    if a!=-1: return a # shortcut for perfect matches
+    a,b = -1,-1
+    if max==1 :
+        a,b = bit_parallel_with_max_1_error(G, H, m)
+    elif max==2 :
+        a,b = bit_parallel_with_max_2_error(G, H, m)
+    if a==1: return b
+    return -1
+
+
+''' replaced with above mmfind()
 # find index of H[1..m] in G[1..n] with up to max mismatches
 
 def mmfind(G,n,H,m,max): # lengths; assume n>m
@@ -155,6 +222,8 @@ def mmfind(G,n,H,m,max): # lengths; assume n>m
       if cnt>max: break
     if cnt<=max: return i
   return -1
+'''
+
 
 def extract_staggered(infile,outfile,vars):
   Tn = vars.prefix
@@ -165,12 +234,19 @@ def extract_staggered(infile,outfile,vars):
 
   #P,Q = 5,10 # 1-based inclusive positions to look for start of Tn prefix
   P,Q = 0,15
+  if vars.barseq_catalog_out!=None: Q = 100 # relax for barseq
 
   vars.tot_tgtta = 0
   vars.truncated_reads = 0
   output = open(outfile,"w")
   tot = 0
   #print infile
+  if vars.barseq_catalog_out!=None:
+    barcodes_file = vars.base+".barseq" # I could define this in vars
+    catalog = open(barcodes_file,"w")
+    barseq1 = "TGCAGGGATGTCCACGAGGTCTCT" # const regions surrounding barcode
+    barseq2 = "CGTACGCTGCAGGTCGACGGCCGG"
+    barseq1len,barseq2len = len(barseq1),len(barseq2)
   for line in open(infile):
     #print line
     line = line.rstrip()
@@ -184,21 +260,30 @@ def extract_staggered(infile,outfile,vars):
     if a>=P and a<=Q:
       gstart,gend = a+lenTn,readlen
       if b!=-1: gend = b; vars.truncated_reads += 1
-      #if gend-gstart<20: continue # too short
-      if gend-gstart<5: continue # too short
+      if gend-gstart<20: continue # too short # I should make this a param
       output.write(header+"\n")
       output.write(line[gstart:gend]+"\n")
       vars.tot_tgtta += 1
+    if vars.barseq_catalog_out!=None:
+      n = max(a,readlen)
+      c = mmfind(line,n,barseq1,barseq1len,vars.mm1) # only have to search as far as Tn prefix
+      d = mmfind(line,n,barseq2,barseq2len,vars.mm1)
+      seq = "XXXXXXXXXXXXXXXXXXXX"
+      if c!=-1 and d!=-1:
+        size = d-c-barseq1len
+        if size>=15 and size<=25: seq = line[c+barseq1len:d]
+      catalog.write(header+"\n")
+      catalog.write(seq+"\n")
+  if vars.barseq_catalog_out!=None: catalog.close()
   output.close()
   if vars.tot_tgtta == 0:
     raise ValueError("Error: Input files did not contain any reads matching prefix sequence with %d mismatches" % vars.mm1)
 
 
-
-
 def message(s):
-  print "[tn_preprocess]",s
-  sys.stdout.flush()
+  #print "[tn_preprocess]",s
+  #sys.stdout.flush()
+  sys.stderr.write("[tn_preprocess] "+s+"\n")
 
 def get_id(line):
   a,b = line.find(":")+1,line.rfind("#")
@@ -210,7 +295,7 @@ def get_id(line):
 def select_reads(goodreads,infile,outfile):
   hash = {}
   for line in open(goodreads):
-    if line[0]=='>': 
+    if line[0]=='>':
       #id = line[line.find(":")+1:line.rfind("#")]
       id = get_id(line)
       hash[id] = 1
@@ -236,7 +321,7 @@ def replace_ids(infile1,infile2,outfile):
     b = g.readline()
     if len(a)<2: break
     if a[0]=='>': header = a
-    else: 
+    else:
       h.write(header)
       h.write(b)
   f.close()
@@ -282,7 +367,7 @@ def template_counts(ref,sam,bcfile,vars):
   genome = read_genome(ref)
   barcodes = {}
 
-  
+
   fil1 = open(bcfile)
   fil2 = open(sam)
 
@@ -295,7 +380,7 @@ def template_counts(ref,sam,bcfile,vars):
   for line in fil2:
     if idx==2: break
     idx+=1
-  
+
   '''
   for line in open(bcfile):
     line = line.rstrip()
@@ -319,12 +404,12 @@ def template_counts(ref,sam,bcfile,vars):
       w = line.split('\t')
       code = samcode(w[1])
       if 'S' in w[5]: continue #elimate softclipped reads
-      if code[6]=="1": # previously checked for for reads1's via w[1]<128 
-        vars.tot_tgtta += 1 
+      if code[6]=="1": # previously checked for for reads1's via w[1]<128
+        vars.tot_tgtta += 1
         if code[2]=="0": vars.r1 += 1
       if code[7]=="1" and code[2]=="0": vars.r2 += 1
       # include "improperly mapped reads, which might just be short frags
-      #if w[1]=="99" or w[1]=="83" or w[1]=="97" or w[1]=="81": 
+      #if w[1]=="99" or w[1]=="83" or w[1]=="97" or w[1]=="81":
       if code[6]=="1" and code[2]=="0" and code[3]=="0": # both reads mapped (proper or not)
         vars.mapped += 1
         readlen = len(w[9])
@@ -369,7 +454,7 @@ def increase_counts(pos,sites, strand):
             sites[pos][4] += 1  #if read has been found before, tally 1 more in R reads
         sites[pos][5] += 1  #if read has been found before, tally 1 more in R reads
         sites[pos][6] += 1  #if read has been found before, tally 1 more in R reads
-    
+
 
 def read_counts(ref,sam,vars):
     genome = read_genome(ref)
@@ -378,7 +463,7 @@ def read_counts(ref,sam,vars):
         if genome[i:i+2]=="TA" or vars.transposon=='Tn5':
           pos = i+1
           sites[pos] = [pos,0,0,0,0,0,0]
-     
+
     hits = {}
     vars.tot_tgtta,vars.mapped = 0,0
     vars.r1 = vars.r2 = 0
@@ -408,7 +493,7 @@ def read_counts(ref,sam,vars):
                     site1 = pos + delta #if on + strand, take column 3 position and add 1bp)
                     if site1 in sites:
                         increase_counts(site1, sites, strand)
-    
+
     results = []
     for key in sorted(sites.keys()):
         results.append(sites[key])
@@ -440,7 +525,7 @@ def driver(vars):
      extract_reads(vars)
 
      run_bwa(vars)
- 
+
      generate_output(vars)
 
   except ValueError as err:
@@ -468,7 +553,7 @@ def uncompress(filename):
 
 def extract_reads(vars):
     message("extracting reads...")
-    
+
     flag = ['','']
     for idx, name in enumerate([vars.fq1, vars.fq2]):
         if idx==1 and vars.single_end==True: continue
@@ -479,11 +564,11 @@ def extract_reads(vars):
                 break
             flag[idx] = 'FASTQ'
             break
-        fil.close() 
+        fil.close()
 
     if vars.fq1.endswith('.gz'):
        vars.fq1 = uncompress(vars.fq1)
-        
+
     if vars.fq2.endswith('.gz'):
        vars.fq2 = uncompress(vars.fq2)
 
@@ -498,9 +583,9 @@ def extract_reads(vars):
       message("creating %s" % vars.trimmed1)
       extract_staggered(vars.reads1,vars.trimmed1,vars)
 
-      return 
+      return
 
-    if(flag[1] == 'FASTQ'):  
+    if(flag[1] == 'FASTQ'):
         message("fastq2reads: %s -> %s" % (vars.fq2,vars.reads2))
         fastq2reads(vars.fq2,vars.reads2,vars.maxreads)
     else:
@@ -610,7 +695,7 @@ def run_bwa(vars):
     if not os.path.exists(vars.ref+".amb"):
       cmd = [vars.bwa, "index", vars.ref]
       bwa_subprocess(cmd, sys.stdout)
-       
+
 
     cmd = [vars.bwa, "aln"]
     if vars.flags.strip():
@@ -626,7 +711,7 @@ def run_bwa(vars):
         bwa_subprocess(cmd, outfile)
 
     else:
-     
+
         cmd = [vars.bwa, "aln"]
         if vars.flags.strip():
             cmd.extend(vars.flags.split(" "))
@@ -665,7 +750,7 @@ def get_read_length(filename):
    fil = open(filename)
    i = 0
    for line in fil:
-      if i == 1: 
+      if i == 1:
          #print "reads1 line: " + line
          return len(line.strip())
       i+=1
@@ -683,7 +768,89 @@ def get_genomic_portion(filename):
    return tot_len/n
 
 
+# return list of (item,cnt) sorted by counts
+
+def popularity(lst):
+  hash = {}
+  for x in lst:
+    if x not in hash: hash[x] = 0
+    hash[x] += 1
+  data = [(hash[x],x) for x in hash.keys()]
+  data.sort(reverse=True)
+  data = [(y,x) for (x,y) in data]
+  return data
+
+
+def create_barseq_catalog(vars):
+  barcodes = {} # headers->barcodes
+  for line in open(vars.base+".barseq"):
+    if line[0]=='>': 
+      header = line.rstrip()[1:]
+      header = header.split()[0] # in case it has a space, which is dropped by bwa in sam file
+    else: barcodes[header] = line.split('\n')[0]
+
+  sites,nreads = {},0
+  for line in open(vars.sam):
+    if line[0]=='@': continue
+    w = line.split('\t')
+    samcode = int(w[1])
+    nreads += 1
+    if samcode in [0,16]: # what about PE reads?
+      header,coord = w[0],int(w[3])
+      # see how I adjust coords in read_counts()
+      strand,delta = 'F',-2
+      if samcode==16: strand,delta = 'R',len(w[9]); 
+      coord += delta
+      if strand=='R': coord *= -1
+      if coord not in sites: sites[coord] = [] # use -co for minus strand
+      sites[coord].append(barcodes[header])
+
+  mapsto = {} # barcodes->sites
+  totbc,maptoTAs = 0,0
+  genome = read_genome(vars.ref)
+  for site,bclist in sites.items():
+    for bc in bclist:
+      if bc not in mapsto: mapsto[bc] = []
+      if 'X' not in bc: 
+        mapsto[bc].append(site)
+        totbc += 1
+        if site<0: site *= -1
+        if genome[site-1:site+1]=="TA": maptoTAs += 1
+
+  # good barcodes are those that are associated with only 1 site (must be TA?)
+  # what about redundant sites like IS elements?
+  goodbc = {}
+  for bc,sites2 in mapsto.items(): 
+    pop = popularity(sites2) # make a table of locations at which the barcode appears
+    #print bc,pop
+    if len(pop)==1: goodbc[bc] = 1
+  #for x in sorted(sites.items()): print x[0],genome[x[0]-1:x[0]+1],popularity(x[1])
+
+  file = open(vars.barseq_catalog_out,"w")  
+  file.write("# Barseq (stats are at the bottom): reads = %s, ref = %s\n" % (vars.fq1,vars.ref))
+  n = len(genome)
+  a,b = 0,0
+  for i in range(n-1):
+    if genome[i:i+2]=="TA":
+      a += 1
+      co = i+1
+      barcodes_pos = [(x,'+') for x in sites.get(co,[])]
+      barcodes_neg = [(x,'-') for x in sites.get(-co,[])]
+      pop = popularity(barcodes_pos+barcodes_neg)
+      if len(pop)>0: b += 1
+      else: file.write("# %s %s\n" %(co,genome[i:i+2]))
+      for (bc,strand),cnt in pop:
+        if bc in goodbc: 
+          file.write("%s %s %s %s %s\n" % (co,genome[i:i+2],strand,bc,cnt))
+  file.write("# total_reads: %s, total_barcodes: %s, map_to_TAs: %s, distinct_bc: %s, unimapped: %s, TA_sites_hit: %s/%s\n" % (nreads,totbc,maptoTAs,len(mapsto.keys()),len(goodbc.keys()),b,a))
+  file.close()
+
+
 def generate_output(vars):
+  if vars.barseq_catalog_out!=None: 
+    message("creating Barseq catalog file: "+vars.barseq_catalog_out)
+    create_barseq_catalog(vars)
+
   message("tabulating template counts and statistics...")
   if vars.single_end==True: counts = read_counts(vars.ref,vars.sam,vars) # return read counts copied as template counts
   else: counts = template_counts(vars.ref,vars.sam,vars.barcodes1,vars)
@@ -720,7 +887,7 @@ def generate_output(vars):
     if Himar1[:-5] in line and Himar1 not in line: misprimed += 1
 
 
-  
+
 
   rcounts = [x[5] for x in counts]
   tcounts = [x[6] for x in counts]
@@ -742,7 +909,7 @@ def generate_output(vars):
     BC_corr = corr([x for x in rcounts if x!=0],[x for x in tcounts if x!=0])
   except ValueError:
     BC_corr = float("nan")
- 
+
 
   read_length = get_read_length(vars.base + ".reads1")
   mean_r1_genomic = get_genomic_portion(vars.base + ".trimmed1")
@@ -799,7 +966,7 @@ def generate_output(vars):
       if '#' in line:
           print line.rstrip()
   infile.close()
-  
+
 #############################################################################
 
 def error(s):
@@ -831,7 +998,7 @@ def set_attributes(vars, attributes_list, override=False):
         else:
             if not hasattr(vars, attr):
                 setattr(vars, attr, value)
-            
+
 
 def set_sassetti_defaults(vars):
     attributes_list = []
@@ -866,7 +1033,7 @@ def set_tn5_defaults(vars):
 
 
 def verify_inputs(vars):
-  
+
     if not os.path.exists(vars.fq1): error("reads1 file not found: "+vars.fq1)
     vars.single_end = False
     if vars.fq2=="": vars.single_end = True
@@ -874,6 +1041,7 @@ def verify_inputs(vars):
     if not os.path.exists(vars.ref): error("reference file not found: "+vars.ref)
     if vars.base == '': error("prefix cannot be empty")
     if vars.fq1 == vars.fq2: error('fastq files cannot be identical')
+    if vars.barseq_catalog_in!=None and vars.barseq_catalog_out!=None: error('barseq catalog input and output files cannot both be defined at the same time')
 
     # If Mme1 protocol, warn that we don't use read2 file
     if vars.protocol.lower() == "mme1" and not vars.single_end:
@@ -902,7 +1070,8 @@ def initialize_globals(vars, args=[], kwargs={}):
     vars.protocol = "Sassetti"
     vars.prefix = "ACTTATCAGCCAACCTGTTA"
     vars.flags = ""
-   
+    vars.barseq_catalog_in = vars.barseq_catalog_out = None
+    
     # Update defaults
     protocol = kwargs.get("protocol", "").lower()
     if protocol:
@@ -935,9 +1104,13 @@ def initialize_globals(vars, args=[], kwargs={}):
         vars.base = kwargs["output"]
     if "mismatches" in kwargs:
         vars.mm1 = int(kwargs["mismatches"])
+    if "barseq_catalog_in" in kwargs:
+        vars.barseq_catalog_in = kwargs["barseq_catalog_in"]
+    if "barseq_catalog_out" in kwargs:
+        vars.barseq_catalog_out = kwargs["barseq_catalog_out"]
     if "flags" in kwargs:
         vars.flags = kwargs["flags"]
-
+    # note: if last flag expected an arg but was end of list, it gets value True ; check for this and report as missing # TRU, 10/28/17
 
 
 def read_config(vars):
@@ -954,6 +1127,8 @@ def read_config(vars):
     if len(w)>=2 and w[0]=='protocol': vars.protocol = " ".join(w[1:])
     if len(w)>=2 and w[0]=='primer': vars.prefix = w[1]
     if len(w)>=2 and w[0]=='flags': vars.flags = " ".join(w[1:])
+    if len(w)>=2 and w[0]=='barseq_catalog_in': vars.barseq_catalog_in = w[1]
+    if len(w)>=2 and w[0]=='barseq_catalog_out': vars.barseq_catalog_out = w[1]
 
 
 def save_config(vars):
@@ -963,17 +1138,17 @@ def save_config(vars):
   f.write("ref %s\n" % vars.ref)
   f.write("bwa %s\n" % vars.bwa)
   f.write("prefix %s\n" % vars.base)
-  f.write("mismatches1 %s\n" % vars.mm1) 
+  f.write("mismatches1 %s\n" % vars.mm1)
   f.write("transposon %s\n" % vars.transposon)
   f.write("protocol %s\n" % vars.protocol)
   f.write("primer %s\n" % vars.prefix)
   f.write("flags %s\n" % vars.flags)
+  if vars.barseq_catalog_in!=None: f.write("barseq_catalog_in %s\n" % vars.barseq_catalog_in)
+  if vars.barseq_catalog_out!=None: f.write("barseq_catalog_out %s\n" % vars.barseq_catalog_out)
   f.close()
 
 def show_help():
-  print 'usage: python PATH/src/tpp.py -bwa <PATH_TO_EXECUTABLE> -ref <REF_SEQ> -reads1 <FASTQ_OR_FASTA_FILE> [-reads2 <FASTQ_OR_FASTA_FILE>] -output <BASE_FILENAME> [-maxreads <N>] [-mismatches <N>] [-flags "<STRING>"] [-tn5|-himar1] [-primer <seq>]'
-    
+  print 'usage: python PATH/src/tpp.py -bwa <EXECUTABLE_WITH_PATH> -ref <REF_SEQ> -reads1 <FASTQ_OR_FASTA_FILE> [-reads2 <FASTQ_OR_FASTA_FILE>] -output <BASE_FILENAME> [-maxreads <N>] [-mismatches <N>] [-flags "<STRING>"] [-tn5|-himar1] [-primer <seq>] [-barseq_catalog_in|_out <file>]'
+
 class Globals:
   pass
-
-
