@@ -15,7 +15,8 @@ import griffin
 import resampling
 import hmm
 import rankproduct
-
+import gi
+import utest
 
 methods = {}
 methods["example"] = example.ExampleAnalysis()
@@ -26,6 +27,8 @@ methods["hmm"] = hmm.HMMAnalysis()
 methods["resampling"] = resampling.ResamplingAnalysis()
 methods["tn5gaps"] = tn5gaps.Tn5GapsAnalysis()
 methods["rankproduct"] = rankproduct.RankProductAnalysis()
+methods["utest"] = utest.UTestAnalysis()
+methods["gi"] = gi.GIAnalysis()
 #methods["mcce"] = mcce.MCCEAnalysis()
 #methods["mcce2"] = mcce2.MCCE2Analysis()
 #methods["motifhmm"] = motifhmm.MotifHMMAnalysis()
