@@ -3,31 +3,47 @@ import sys
 
 try:
     import wx
+    WX_VERSION = int(wx.version()[0])
     hasWx = True
-    #Check if wx is the newest 3.0+ version:
-    try:
-        from wx.lib.pubsub import pub
-        pub.subscribe
-        newWx = True
-    except AttributeError as e:
-        from wx.lib.pubsub import Publisher as pub
-        newWx = False
+
 except Exception as e:
     hasWx = False
-    newWx = False
-    
+    WX_VERSION = 0
+    print "EXCEPTION:", str(e)
+
+if hasWx:
+    import wx.xrc
+    from wx.lib.buttons import GenBitmapTextButton
+
+    #Imports depending on version:
+    if WX_VERSION == 2:
+        from wx.lib.pubsub import Publisher as pub
+
+    if WX_VERSION == 3:
+        from wx.lib.pubsub import pub
+        pub.subscribe
+
+    if WX_VERSION == 4:
+        from wx.lib.pubsub import pub
+        pub.subscribe
+        import wx.adv
+
+   
 import traceback
 import datetime
 import pytransit.transit_tools as transit_tools
 
 file_prefix = "[FileDisplay]"
 
+
 class InvalidArgumentException(Exception):
     def __init__(self, message):
 
         # Call the base class constructor with the parameters it needs
         super(InvalidArgumentException, self).__init__(message)
 
+#
+
 if hasWx:
     class InfoIcon(wx.StaticBitmap):
         def __init__(self, panel, flag, bmp=None, tooltip=""):
@@ -44,56 +60,76 @@ class TransitGUIBase:
         self.wxobj = None
         self.short_name = "TRANSIT"
         self.long_name = "TRANSIT"
+        self.short_desc = "TRANSIT - Short Description"
+        self.long_desc =  "TRANSIT - Long Description"
+
+#
 
     def status_message(self, text, time=-1):
         #TODO: write docstring
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage, "status", msg=(self.short_name, text, time))
             else:
                 wx.CallAfter(pub.sendMessage, "status", (self.short_name, text, time))
             wx.Yield()
 
+#
+
     def console_message(self, text):
         #TODO: write docstring
         sys.stdout.write("[%s] %s\n" % (self.short_name, text))
 
+#
+
     def console_message_inplace(self, text):
         #TODO: write docstring
         sys.stdout.write("[%s] %s   \r" % (self.short_name, text) )
         sys.stdout.flush()
 
+#
+
     def transit_message(self, text):
         #TODO: write docstring
         self.console_message(text)
         self.status_message(text)
 
+#
+
     def transit_message_inplace(self, text):
         #TODO: write docstring
         self.console_message_inplace(text)
         self.status_message(text)
 
+#
 
     def transit_error(self,text):
         self.transit_message(text)
         if self.wxobj:
             transit_tools.ShowError(text)
 
+#
+
     def transit_warning(self,text):
         self.transit_message(text)
         if self.wxobj:
             transit_tools.ShowWarning(text)
 
+#
 
 class TransitFile (TransitGUIBase):
     #TODO write docstring
 
+#
+
     def __init__(self, identifier="#Unknown", colnames=[]):
         #TODO write docstring
         TransitGUIBase.__init__(self)
         self.identifier = identifier
         self.colnames = colnames
 
+#
+
     def getData(self, path, colnames):
         #TODO write docstring
         row = 0
@@ -116,15 +152,20 @@ class TransitFile (TransitGUIBase):
             row+=1
         return data
 
+#
+
     def getHeader(self, path):
         #TODO write docstring
         return "Generic Transit File Type."
 
+#
 
     def getMenus(self):
         menus = [("Display in Track View", self.displayInTrackView)]
         return menus
 
+#
+
     def displayInTrackView(self, displayFrame, event):
 
         #print "Self:", self
@@ -137,6 +178,8 @@ class TransitFile (TransitGUIBase):
         except Exception as e:
             print file_prefix, "Error occurred: %s" % e 
 
+#
+
 class AnalysisGUI:
     
     def __init__(self):
@@ -145,16 +188,22 @@ class AnalysisGUI:
         self.LABELSIZE = (100,-1)
         self.WIDGETSIZE = (100,-1)
 
+#
+
     def Hide(self):
         self.panel.Hide()
 
+#
+
     def Show(self):
         self.panel.Show()
 
+#
+
     def Enable(self):
         self.panel.Enable()
 
-
+#
 
     def definePanel(self, wxobj):
         #TODO: write docstring
@@ -164,8 +213,8 @@ class AnalysisGUI:
 
         Section = wx.BoxSizer( wx.VERTICAL )
 
-        Label = wx.StaticText(wPanel, id=wx.ID_ANY, label=str("Options"), pos=wx.DefaultPosition, size=wx.DefaultSize, style=0 )
-        Label.Wrap( -1 )
+        Label = wx.StaticText(wPanel, id=wx.ID_ANY, label=str("Method Options"), pos=wx.DefaultPosition, size=(130, -1), style=0 )
+        Label.SetFont( wx.Font( 10, wx.DEFAULT, wx.NORMAL, wx.BOLD) )
         Section.Add( Label, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
         
         Sizer1 = wx.BoxSizer( wx.HORIZONTAL )
@@ -182,6 +231,8 @@ class AnalysisGUI:
         Button.Bind( wx.EVT_BUTTON, self.wxobj.RunMethod )
         self.panel = wPanel
 
+#
+
     def defineTextBox(self, panel, labelText="", widgetText="", tooltipText="", labSize=None, widgetSize=None):
         if not labSize: labSize = self.LABELSIZE
         if not widgetSize: widgetSize = self.WIDGETSIZE
@@ -195,6 +246,8 @@ class AnalysisGUI:
         sizer.Add(InfoIcon(panel, wx.ID_ANY, tooltip=tooltipText), 0, wx.ALIGN_CENTER_VERTICAL, 5 )
         return (label, textBox, sizer)
 
+#
+
     def defineChoiceBox(self, panel, labelText="", widgetChoice=[""], tooltipText="", labSize=None, widgetSize=None):
         if not labSize: labSize = self.LABELSIZE
         if not widgetSize: widgetSize = self.WIDGETSIZE
@@ -209,6 +262,8 @@ class AnalysisGUI:
         sizer.Add(InfoIcon(panel, wx.ID_ANY, tooltip=tooltipText), 0, wx.ALIGN_CENTER_VERTICAL, 5 )
         return (label, choiceBox, sizer)
 
+#
+
     def defineCheckBox(self, panel, labelText="", widgetCheck=False, tooltipText="", widgetSize=None):
         if not widgetSize: widgetSize = self.WIDGETSIZE
         sizer = wx.BoxSizer( wx.HORIZONTAL )
@@ -218,33 +273,40 @@ class AnalysisGUI:
         sizer.Add(InfoIcon(panel, wx.ID_ANY, tooltip=tooltipText), 0, wx.ALIGN_CENTER_VERTICAL, 5 )
         return (checkBox, sizer)
 
+#
 
 class AnalysisMethod:
     '''
     Basic class for analysis methods. Inherited by SingleMethod and ComparisonMethod.
     '''
     
-    def __init__(self, short_name, long_name, description, output, annotation_path, wxobj=None):
+    def __init__(self, short_name, long_name, short_desc, long_desc, output, annotation_path, wxobj=None):
         self.short_name = short_name
         self.long_name = long_name
-        self.description = description
+        self.short_desc = short_desc
+        self.long_desc = long_desc
         self.output = output
         self.annotation_path = annotation_path
 
-        self.newWx = newWx
+        self.WX_VERSION = WX_VERSION 
         self.wxobj = wxobj
 
+#
 
     @classmethod 
     def fromGUI(self, wxobj):
         #TODO: write docstring
         raise NotImplementedError
 
+#
+
     @classmethod
     def fromargs(self, rawargs):
         #TODO: write docstring
         raise NotImplementedError
 
+#
+
     @classmethod
     def fromconsole(self):
         #TODO: write docstring
@@ -270,15 +332,20 @@ class AnalysisMethod:
             print self.usage_string()
         sys.exit() 
 
+#
+
     @classmethod
     def usage_string(self):
         #TODO: write docstring
         raise NotImplementedError
 
+#
+
     def Run(self):
         #TODO write docstring
         raise NotImplementedError
 
+#
 
     def print_members(self):
         #TODO: write docstring
@@ -286,6 +353,7 @@ class AnalysisMethod:
         for m in members:
             print "%s = %s" % (m, getattr(self, m))
 
+#
 
     def add_file(self, path=None, filetype=None):
 
@@ -298,73 +366,91 @@ class AnalysisMethod:
         data = {"path":path, "type":filetype, "date": datetime.datetime.today().strftime("%B %d, %Y %I:%M%p")}
 
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage, "file", data=data)
             else:
                 wx.CallAfter(pub.sendMessage, "file", data)
 
+#
+
     def finish(self):
         #TODO: write docstring
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage,"finish", msg=self.short_name.lower())
             else:
                 wx.CallAfter(pub.sendMessage,"finish", self.short_name.lower())
 
+#
+
     def progress_update(self, text, count):
         #TODO: write docstring
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage, "progress", msg=(self.short_name, count))
             else:
                 wx.CallAfter(pub.sendMessage, "progress", (self.short_name, count))
             wx.Yield()
 
+        self.transit_message_inplace(text)
+#
+
     def progress_range(self, count):
         #TODO: write docstring
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage, "progressrange", msg=count)
             else:
                 wx.CallAfter(pub.sendMessage, "progressrange", count)
             wx.Yield()
 
-        
+#       
 
     def status_message(self, text, time=-1):
         #TODO: write docstring
         if self.wxobj:
-            if newWx:
+            if WX_VERSION > 2:
                 wx.CallAfter(pub.sendMessage, "status", msg=(self.short_name, text, time))
             else:
                 wx.CallAfter(pub.sendMessage, "status", (self.short_name, text, time))
             wx.Yield()
 
+#
+
     def console_message(self, text):
         #TODO: write docstring
         sys.stdout.write("[%s] %s\n" % (self.short_name, text))
 
+#
+
     def console_message_inplace(self, text):
         #TODO: write docstring
         sys.stdout.write("[%s] %s   \r" % (self.short_name, text) )
         sys.stdout.flush()
 
+#
+
     def transit_message(self, text):
         #TODO: write docstring
         self.console_message(text)
         self.status_message(text)
 
+#
+
     def transit_message_inplace(self, text):
         #TODO: write docstring
         self.console_message_inplace(text)
         self.status_message(text)
 
+#
 
     def transit_error(self,text):
         self.transit_message(text)
         if self.wxobj:
             transit_tools.ShowError(text)
 
+#
+
     def transit_warning(self,text):
         self.transit_message(text)
         if self.wxobj:
@@ -376,8 +462,9 @@ class SingleConditionMethod(AnalysisMethod):
     Class to be inherited by analysis methods that determine essentiality in a single condition (e.g. Gumbel, Binomial, HMM).
     '''
 
-    def __init__(self, short_name, long_name, description, ctrldata, annotation_path, output, replicates="Sum", normalization=None, LOESS=False, ignoreCodon=True, NTerminus=0.0, CTerminus=0.0, wxobj=None):
-        AnalysisMethod.__init__(self, short_name, long_name, description, output, annotation_path, wxobj)
+    def __init__(self, short_name, long_name, short_desc, long_desc, ctrldata, annotation_path, output, replicates="Sum", normalization=None, LOESS=False, ignoreCodon=True, NTerminus=0.0, CTerminus=0.0, wxobj=None):
+        AnalysisMethod.__init__(self, short_name, long_name, short_desc, long_desc, output, 
+            annotation_path, wxobj)
         self.ctrldata = ctrldata
         self.replicates = replicates
         self.normalization = normalization
@@ -386,15 +473,16 @@ class SingleConditionMethod(AnalysisMethod):
         self.NTerminus = NTerminus
         self.CTerminus = CTerminus
 
-
+#
 
 class DualConditionMethod(AnalysisMethod):
     '''
     Class to be inherited by analysis methods that determine changes in essentiality between two conditions (e.g. Resampling, DEHMM).
     '''
 
-    def __init__(self, short_name, long_name, description, ctrldata, expdata, annotation_path, output, normalization, replicates="Sum", LOESS=False, ignoreCodon=True, NTerminus=0.0, CTerminus=0.0, wxobj=None):
-        AnalysisMethod.__init__(self, short_name, long_name, description, output, annotation_path, wxobj)
+    def __init__(self, short_name, long_name, short_desc, long_desc, ctrldata, expdata, annotation_path, output, normalization, replicates="Sum", LOESS=False, ignoreCodon=True, NTerminus=0.0, CTerminus=0.0, wxobj=None):
+        AnalysisMethod.__init__(self, short_name, long_name, short_desc, long_desc, 
+            output, annotation_path, wxobj)
         self.ctrldata = ctrldata
         self.expdata = expdata
         self.normalization = normalization
@@ -404,36 +492,67 @@ class DualConditionMethod(AnalysisMethod):
         self.NTerminus = NTerminus
         self.CTerminus = CTerminus
 
+#
 
+class QuadConditionMethod(AnalysisMethod):
+    '''
+    Class to be inherited by analysis methods that determine changes in essentiality between four conditions (e.g. GI).
+    '''
+
+    def __init__(self, short_name, long_name, short_desc, long_desc, ctrldataA, ctrldataB, expdataA, expdataB, annotation_path, output, normalization, replicates="Sum", LOESS=False, ignoreCodon=True, NTerminus=0.0, CTerminus=0.0, wxobj=None):
+        AnalysisMethod.__init__(self, short_name, long_name, short_desc, long_desc, 
+            output, annotation_path, wxobj)
+        self.ctrldataA = ctrldataA
+        self.ctrldataB = ctrldataB
+        self.expdataA = expdataA
+        self.expdataB = expdataB
+        self.normalization = normalization
+        self.replicates = replicates
+        self.LOESS = LOESS
+        self.ignoreCodon = ignoreCodon
+        self.NTerminus = NTerminus
+        self.CTerminus = CTerminus
+
+#
 
 class TransitAnalysis:
-    def __init__(self, sn, ln, desc, tn, method_class=AnalysisMethod, gui_class=AnalysisGUI, filetypes=[TransitFile]):
+    def __init__(self, sn, ln, short_desc, long_desc, tn, method_class=AnalysisMethod, gui_class=AnalysisGUI, filetypes=[TransitFile]):
         self.short_name = sn
         self.long_name = ln
-        self.description = desc
+        self.short_desc = short_desc
+        self.long_desc = long_desc
         self.transposons = tn
         self.method = method_class
         self.gui = gui_class()
         self.filetypes = filetypes
 
+#
 
     def __str__(self):
         return """Analysis Method:
     Short Name:  %s
     Long Name:   %s
-    Description: %s
+    Short Desc:  %s
+    Long Desc:   %s
     Method:      %s
-    GUI:         %s""" % (self.short_name, self.long_name, self.description, self.method, self.gui)
+    GUI:         %s""" % (self.short_name, self.long_name, self.short_desc, self.long_desc, self.method, self.gui)
 
+#
 
     def fullname(self):
-        return "[%s]  -  %s" % (self.short_name, self.long_name)
+        return "[%s]  -  %s" % (self.short_name, self.short_desc)
+
+#
 
     def getInstructionsText(self):
         return ""
 
+#
+
     def getDescriptionText(self):
-        return self.description 
+        return self.long_desc
+
+#
 
     def getTransposonsText(self):
         if len(self.transposons) == 0:
@@ -446,6 +565,7 @@ class TransitAnalysis:
             return "Intended for " + ", ".join(self.transposons[:-1]) + ", and " + self.transposons[-1]
     
 
+#
 
 if __name__ == "__main__":
     pass
