@@ -2,18 +2,31 @@ import sys
 
 try:
     import wx
+    WX_VERSION = int(wx.version()[0])
     hasWx = True
-    #Check if wx is the newest 3.0+ version:
-    try:
-        from wx.lib.pubsub import pub
-        pub.subscribe
-        newWx = True
-    except AttributeError as e:
-        from wx.lib.pubsub import Publisher as pub
-        newWx = False
+
 except Exception as e:
     hasWx = False
-    newWx = False
+    WX_VERSION = 0
+    print "EXCEPTION:", str(e)
+
+if hasWx:
+    import wx.xrc
+    from wx.lib.buttons import GenBitmapTextButton
+
+    #Imports depending on version:
+    if WX_VERSION == 2:
+        from wx.lib.pubsub import Publisher as pub
+
+    if WX_VERSION == 3:
+        from wx.lib.pubsub import pub
+        pub.subscribe
+
+    if WX_VERSION == 4:
+        from wx.lib.pubsub import pub
+        pub.subscribe
+        import wx.adv
+
 
 import os
 import time
@@ -34,8 +47,9 @@ import pytransit.stat_tools as stat_tools
 ############# GUI ELEMENTS ##################
 
 short_name = "gumbel"
-long_name = "Bayesian analysis of essentiality based on long gaps."
-description = """Bayesian methods of analyzing longest runs of non-insertions in a row. Estimates the parameters using the MCMC sampling, and estimates posterior probabilities of essentiality. 
+long_name = "Gumbel"
+short_desc = "Bayesian analysis of essentiality based on long gaps."
+long_desc = """Bayesian methods of analyzing longest runs of non-insertions in a row. Estimates the parameters using the MCMC sampling, and estimates posterior probabilities of essentiality. 
 
 Reference: DeJesus et al. (2013; Bioinformatics)"""
 transposons = ["himar1"]
@@ -46,7 +60,7 @@ columns = ["Orf","Name","Desc","k","n","r","s","zbar", "Call"]
 
 class GumbelAnalysis(base.TransitAnalysis):
     def __init__(self):
-        base.TransitAnalysis.__init__(self, short_name, long_name, description, transposons, GumbelMethod, GumbelGUI, [GumbelFile])
+        base.TransitAnalysis.__init__(self, short_name, long_name, short_desc, long_desc, transposons, GumbelMethod, GumbelGUI, [GumbelFile])
 
 
 ################## FILE ###################
@@ -88,8 +102,9 @@ class GumbelGUI(base.AnalysisGUI):
 
         gumbelSection = wx.BoxSizer( wx.VERTICAL )
 
-        gumbelLabel = wx.StaticText( gumbelPanel, wx.ID_ANY, u"Gumbel Options", wx.DefaultPosition, wx.DefaultSize, 0 )
-        gumbelLabel.Wrap( -1 )
+        gumbelLabel = wx.StaticText( gumbelPanel, wx.ID_ANY, u"Gumbel Options", wx.DefaultPosition, (130,20))
+    
+        gumbelLabel.SetFont( wx.Font( 10, wx.DEFAULT, wx.NORMAL, wx.BOLD) )
         gumbelSection.Add( gumbelLabel, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL, 5 )
 
         mainSizer1 = wx.BoxSizer( wx.VERTICAL )
@@ -140,6 +155,8 @@ class GumbelGUI(base.AnalysisGUI):
 
 ########## METHOD #######################
 
+EXACT = 20
+
 ALPHA = 1
 BETA = 1
 class GumbelMethod(base.SingleConditionMethod):
@@ -162,7 +179,7 @@ class GumbelMethod(base.SingleConditionMethod):
                 NTerminus=0.0,
                 CTerminus=0.0, wxobj=None):
 
-        base.SingleConditionMethod.__init__(self, short_name, long_name, description, ctrldata, annotation_path, output_file, replicates=replicates, normalization=normalization, LOESS=LOESS, NTerminus=NTerminus, CTerminus=CTerminus, wxobj=wxobj)
+        base.SingleConditionMethod.__init__(self, short_name, long_name, short_desc, long_desc, ctrldata, annotation_path, output_file, replicates=replicates, normalization=normalization, LOESS=LOESS, NTerminus=NTerminus, CTerminus=CTerminus, wxobj=wxobj)
         self.samples = samples
         self.burnin = burnin
         self.trim = trim
@@ -170,8 +187,7 @@ class GumbelMethod(base.SingleConditionMethod):
       
 
         self.cache_nn = {}
- 
-
+        self.cache_expruns = {}
 
     @classmethod
     def fromGUI(self, wxobj):
@@ -365,6 +381,10 @@ class GumbelMethod(base.SingleConditionMethod):
         
         SIG = numpy.array([self.sigmoid(g.s, g.t) * scipy.stats.norm.pdf(g.r, mu_r*g.s, sigma_r) for g in G if self.good_orf(g)])
 
+#        idxG,idxN = -1,0
+#        for i in range(len(G)):
+#          if G[i].name=="glf": idxG = i
+#          if ii_good[i]==True: idxN += 1 # could do sum(ii_good[:idxG])
 
         i = 1; count = 0;
         while i < self.samples:
@@ -382,7 +402,7 @@ class GumbelMethod(base.SingleConditionMethod):
                 # Z
                 Z = self.sample_Z(phi_new, w1, N, R, S, T, mu_s, sigma_s, SIG)
             
-            # w1
+                # w1
                 N_ESS = sum(Z == 1)
                 w1 = scipy.stats.beta.rvs(N_ESS + ALPHA_w, N_GOOD - N_ESS + BETA_w)
             
@@ -393,19 +413,20 @@ class GumbelMethod(base.SingleConditionMethod):
                     phi_sample[i] = phi_new
                     Z_sample[:,i] = Z
                     i+=1
+
             except ValueError as e:
                 self.transit_message("Error: %s" % e) 
                 self.transit_message("This is likely to have been caused by poor data (e.g. too sparse).") 
                 self.transit_message("If the density of the dataset is too low, the Gumbel method will not work.") 
                 self.transit_message("Quitting.") 
                 return
-                
+
+#            print i,phi_new,w1,G[idxG].name,N[idxN],R[idxN],Z[idxN]
             
             phi_old = phi_new
             #Update progress
-            text = "Running Gumbel Method... %2.0f%%" % (100.0*(count+1)/(self.samples+self.burnin))
+            text = "Running Gumbel Method... %5.1f%%" % (100.0*(count+1)/(self.samples+self.burnin))
             self.progress_update(text, count)
-            self.transit_message_inplace(text)
 
 
         ZBAR = numpy.apply_along_axis(numpy.mean, 1, Z_sample)
@@ -476,28 +497,44 @@ class GumbelMethod(base.SingleConditionMethod):
     def good_orf(self, gene):
         return (gene.n >= 3 and gene.t >= 150)
 
+    def ExpectedRuns_cached(self,n,q):
+        if (n,q) not in self.cache_expruns: self.cache_expruns[(n,q)] = tnseq_tools.ExpectedRuns(n,q)
+        return self.cache_expruns[(n,q)]
+
     def classify(self, n,r,p):
         if n == 0: return 0
-        q = 1-p; B = 1/math.log(1/p); u = math.log(n*q,1/p);
-        pval = 1 - scipy.stats.gumbel_r.cdf(r,u,B)
+        q = 1-p; B = 1/math.log(1/p); u = math.log(n*q,1/p) 
+        BetaGamma = B*tnseq_tools.getGamma()
+        if n<EXACT: # estimate more accurately based on expected run len, using exact calc for small genes
+          exprun = self.ExpectedRuns_cached(n,p)
+          u = exprun-BetaGamma # u is mu of Gumbel (mean=mu+gamma*beta); matching of moments
+        pval = 1 - scipy.exp(scipy.stats.gumbel_r.logcdf(r,u,B))
         if pval < 0.05: return(1)
         else: return(0)
 
-    def F_non(self, p, N, R):
-        q = 1.0 - p
+    def F_non(self, p, N, R): # pass in P_nonins as p
+        q = 1.0 - p; 
+        BetaGamma = tnseq_tools.getGamma()/math.log(1/p)
         total = numpy.log(scipy.stats.beta.pdf(p,ALPHA,BETA))
         mu = numpy.log(N*q) / numpy.log(1/p)
+        for i in range(len(N)): # estimate more accurately based on expected run len, using exact calc for small genes
+          if N[i]<EXACT: mu[i] = self.ExpectedRuns_cached(int(N[i]),p)-BetaGamma
         sigma = 1/math.log(1/p);
-        total+= numpy.sum(numpy.log(scipy.stats.gumbel_r.pdf(R, mu, sigma)))
+        #for i in range(len(N)): print '\t'.join([str(x) for x in N[i],R[i],self.ExpectedRuns_cached(int(N[i]),q),mu[i],scipy.stats.gumbel_r.pdf(R[i], mu[i], sigma)])
+        total += numpy.sum(scipy.stats.gumbel_r.logpdf(R, mu, sigma))
         return(total)
     
     def sample_Z(self, p, w1, N, R, S, T, mu_s, sigma_s, SIG):
         G = len(N)
         q = 1.0-p
-        mu = numpy.log(N*q) / numpy.log(1.0/p)
+        BetaGamma = tnseq_tools.getGamma()/math.log(1/p)
+        mu = numpy.log(N*q) / numpy.log(1/p)
+        for i in range(len(N)): # estimate more accurately based on expected run len, using exact calc for small genes
+          if N[i]<EXACT: mu[i] = self.ExpectedRuns_cached(int(N[i]),p)-BetaGamma
         sigma = 1.0/math.log(1.0/p);
-        h0 = ((scipy.stats.gumbel_r.pdf(R,mu,sigma)) * scipy.stats.norm.pdf(S, mu_s*R, sigma_s)  * (1-w1))
+        h0 = ((scipy.exp(scipy.stats.gumbel_r.logpdf(R,mu,sigma))) * scipy.stats.norm.pdf(S, mu_s*R, sigma_s)  * (1-w1))
         h1 = SIG * w1
+        h1 += 1e-10; h0 += 1e-10 # to prevent div-by-zero; if neither class is probable, p(z1) should be ~0.5
         p_z1 = h1/(h0+h1)
         return scipy.stats.binom.rvs(1, p_z1, size=G)
 
